{ pkgs ? import <nixpkgs> {} }:

let
  crossPkgs = pkgs.pkgsCross.aarch64-multiplatform;
in pkgs.mkShell rec {
	buildInputs = with pkgs; [
      clang
      llvmPackages_17.bintools
      rustup
	  qemu
      grub2
      libisoburn
      bison
      flex
      gmp
      libmpc
      mpfr
      texinfo
      isl
    ];
    RUSTC_VERSION = "nightly";
    hardeningDisable = [ "all" ];
    # https://github.com/rust-lang/rust-bindgen#environment-variables
    LIBCLANG_PATH = pkgs.lib.makeLibraryPath [ pkgs.llvmPackages_latest.libclang.lib ];
	shellHook = ''
      rustup default nightly
	  rustup component add llvm-tools-preview
	  rustup component add rust-analyzer
	  rustup target add x86_64-unknown-none
      export PATH=$PATH:''${CARGO_HOME:-~/.cargo}/bin
      export PATH=$PATH:''${RUSTUP_HOME:-~/.rustup}/toolchains/nightly-x86_64-unknown-linux-gnu/bin/
      export PATH=$PATH:$PWD/opt/bin
      '';
    # Add precompiled library to rustc search path
    RUSTFLAGS = (builtins.map (a: ''-L ${a}/lib'') [
      # add libraries here (e.g. pkgs.libvmi)
    ]);
    # Add glibc, clang, glib, and other headers to bindgen search path
    BINDGEN_EXTRA_CLANG_ARGS = 
    # Includes normal include path
    (builtins.map (a: ''-I"${a}/include"'') [
      # add dev libraries here (e.g. pkgs.libvmi.dev)
      pkgs.glibc.dev 
    ])
    # Includes with special directory paths
    ++ [
      ''-I"${pkgs.llvmPackages_latest.libclang.lib}/lib/clang/${pkgs.llvmPackages_latest.libclang.version}/include"''
      ''-I"${pkgs.glib.dev}/include/glib-2.0"''
      ''-I${pkgs.glib.out}/lib/glib-2.0/include/''
    ];
}
